/* 
* classAd9837.cpp
*
* Created: 30/04/2021 20:14:28
* Author: Ledo
*/


#include "classAd9837.h"

// default constructor
classAd9837::classAd9837()
{
} //classAd9837

// default destructor
classAd9837::~classAd9837()
{
} //~classAd9837

/***************************************************************************//**
 * @brief Writes the value to a register.
 *
 * @param -  regValue - The value to write to the register.
 *
 * @return  None.    
*******************************************************************************/
void classAd9837::SetRegisterValue(uint16_t data)
{
	  SPIe.spi_init(SPI_MODE_2_gc);
	  chip_select();
	  _delay_us(2);
	  SPIe.transfer(data >> 8);
	  SPIe.transfer(data & 0xFF);
	  chip_deselect();
	  SPIe.spi_init(SPI_MODE_0_gc);

}

/***************************************************************************//**
 * @brief Sets the Reset bit of the AD9837.
 *
 * @return None.
*******************************************************************************/
void classAd9837::Reset(void)
{
    SetRegisterValue(AD9837_REG_CMD | AD9837_RESET);
}

/***************************************************************************//**
 * @brief Clears the Reset bit of the AD9837.
 *
 * @return None.
*******************************************************************************/
void classAd9837::ClearReset(void)
{
	SetRegisterValue(AD9837_REG_CMD);
}

/***************************************************************************//**
 * @brief Writes to the frequency registers.
 *
 * @param -  reg - Frequence register to be written to.
 * @param -  val - The value to be written.
 *
 * @return  None.    
*******************************************************************************/
void classAd9837::SetFrequency(uint32_t val)
{
	uint16_t freqHi;
	uint16_t freqLo;
	uint32_t FW;
	FW = ((uint64_t)val * F_RESOLUTION) / M_CLOCK;
	freqHi = AD9837_REG_FREQ0 | ((FW & 0xFFFC000) >> 14) ;
	freqLo = AD9837_REG_FREQ0 |  (FW & 0x3FFF);
	SetRegisterValue(AD9837_B28);
	SetRegisterValue(freqLo);
	SetRegisterValue(freqHi);
}

/***************************************************************************//**
 * @brief Writes to the phase registers.
 *
 * @param -  reg - Phase register to be written to.
 * @param -  val - The value to be written.
 *
 * @return  None.    
*******************************************************************************/
void classAd9837::SetPhase(uint16_t reg, uint16_t val)
{
	uint16_t phase = reg;
	phase |= val;
	SetRegisterValue(phase);
}
/***************************************************************************//**
 * @brief Selects the Frequency,Phase and Waveform type.
 *
 * @param -  freq  - Frequency register used.
 * @param -  phase - Phase register used.
 * @param -  type  - Type of waveform to be output.
 *
 * @return  None.    
*******************************************************************************/
void classAd9837::Setup(uint16_t freq,uint16_t phase,uint16_t type)
{
	uint16_t val = 0;
	
	val = freq | phase | type;
	SetRegisterValue(val);
}

/***************************************************************************//**
 * @brief Sets the type of waveform to be output.
 *
 * @param -  type - type of waveform to be output.
 *
 * @return  None.    
*******************************************************************************/
void classAd9837::SetWave(uint16_t type)
{
	SetRegisterValue(type);
}
