/* 
* classAd9837.h
*
* Created: 30/04/2021 20:14:29
* Author: Ledo
*/


#ifndef __CLASSAD9837_H__
#define __CLASSAD9837_H__

#include <avr/io.h>
#include "classSPI.h"
#include "tipos.h"
#define F_CPU (32000000UL)
#include <util/delay.h>

//Pines de control AD9837
#define AD9837_CS_PIN_MASK                 PIN3_bm  //PORTC




/******************************************************************************/
/* AD9837                                                                    */
/******************************************************************************/
/* Registers */

#define AD9837_REG_CMD		(0 << 14)
#define AD9837_REG_FREQ0	0x4000
#define AD9837_REG_FREQ1	(2 << 14)
#define AD9837_REG_PHASE0	(6 << 13)
#define AD9837_REG_PHASE1	(7 << 13)

/* Command Control Bits */

#define AD9837_B28			(1 << 13)
#define AD9837_HLB			(1 << 12)
#define AD9837_FSEL0		(0 << 11)
#define AD9837_FSEL1		(1 << 11)
#define AD9837_PSEL0		(0 << 10)
#define AD9837_PSEL1		(1 << 10)
#define AD9837_PIN_SW		(1 << 9)
#define AD9837_RESET		(1 << 8)
#define AD9837_SLEEP1		(1 << 7)
#define AD9837_SLEEP12		(1 << 6)
#define AD9837_OPBITEN		(1 << 5)
#define AD9837_SIGN_PIB		(1 << 4)
#define AD9837_DIV2			(1 << 3)
#define AD9837_MODE			(1 << 1)

#define AD9837_OUT_SINUS	((0 << 5) | (0 << 1) | (0 << 3))
#define AD9837_OUT_TRIANGLE	((0 << 5) | (1 << 1) | (0 << 3))
#define AD9837_OUT_MSB		((1 << 5) | (0 << 1) | (1 << 3))
#define AD9837_OUT_MSB2		((1 << 5) | (0 << 1) | (0 << 3))

#define M_CLOCK            16000000UL     //16 MHz Master Clock
#define F_RESOLUTION       0x10000000UL   //28 bits

extern classSPI SPIe;

class classAd9837
{
//variables
public:
   int32_t Frequency; //Hz
protected:
private:

//functions
public:
	classAd9837();
	~classAd9837();
	/* Sets the Reset bit of the AD9837. */
	void Reset(void);
	/* Clears the Reset bit of the AD9837. */
	void ClearReset(void);
	/* Writes to the frequency registers. */
	void SetFrequency(uint32_t val);
	/* Writes to the phase registers. */
	void SetPhase(uint16_t reg, uint16_t val);
	/* Selects the Frequency,Phase and Waveform type. */
	void Setup(uint16_t freq,uint16_t phase,uint16_t type);
	void SetWave(uint16_t type);
protected:
private:
	classAd9837( const classAd9837 &c );
	classAd9837& operator=( const classAd9837 &c );
	#define chip_select()     {PORTC.OUTCLR = AD9837_CS_PIN_MASK;}
	#define chip_deselect()   {PORTC.OUTSET = AD9837_CS_PIN_MASK;}
	/* Writes the value to a register. */
	void SetRegisterValue(uint16_t data);

}; //classAd9837

#endif //__CLASSAD9837_H__
